class InvalidStatusError(Exception):
    pass


class NoRoutesError(Exception):
    pass


class RouteClassError(Exception):
    pass


class FormDataError(Exception):
    pass


class MultipartFormDataError(Exception):
    pass


class TemplatePathError(Exception):
    pass


class StaticFileError(Exception):
    pass
